@@ -10,7 +10,7 @@ require HTML::DOM::Element;
 require HTML::DOM::NodeList::Magic;
 #require HTML::DOM::Collection::Elements;
 
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 use overload fallback => 1,
@@ -28,33 +28,37 @@ my %elem_elems = (
 	select   => 1,
 	textarea => 1,
 );
-sub elements { # ~~~ I need to make this cache the resulting collection obj
+sub elements {
 	my $self = shift;
-	if (wantarray) {
-		return grep $elem_elems{tag $_}, $self->descendants;
-	}
-	else {
+	my $collection = $self->{_HTML_DOM_elems} ||= do {
 		my $collection = HTML::DOM::Collection::Elements->new(
 		my $list = HTML::DOM::NodeList::Magic->new(
-		    sub {
+		   sub {
 		     grep($elem_elems{tag $_}, $self->descendants),
-		     @{ $self->{_HTML_DOM_elems}||[] }
-		    }
+		     @{ $self->{_HTML_DOM_mg_elems}||[] }
+		    } 
 		));
 		$self->ownerDocument-> _register_magic_node_list($list);
 		$collection;
+	};
+	weaken $self;
+	if (wantarray) {
+		@$collection
+	}
+	else {
+		$collection;
 	}
 }
 sub add_element { # helper routine that formies use to add themselves to
  my $self = shift;  # the elements list
- push @{ $self->{_HTML_DOM_elems} ||= [] }, shift
+ push @{ $self->{_HTML_DOM_mg_elems} ||= [] }, shift
   if $elem_elems{ $_[0]->tag };
  return;
 }
 sub remove_element { # and this is how formies remove themselves when they
  my $self = shift;   # get moved around the DOM
  my $removee = shift;
- @{ $self->{_HTML_DOM_elems} }
+ @{ $self->{_HTML_DOM_mg_elems} }
   = grep $_ != $removee, @{ $self->{_HTML_DOM_elems} ||= [] }
 }
 
@@ -391,7 +395,7 @@ package HTML::DOM::NodeList::Radio; # solely for HTML::Form compatibility
 use Carp 'croak';
 require HTML::DOM::NodeList;
 
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::NodeList';
 
 sub type { 'radio' }
@@ -463,7 +467,7 @@ use warnings;
 
 use Scalar::Util 'weaken';
 
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 
 require HTML::DOM::Collection;
 our @ISA = 'HTML::DOM::Collection';
@@ -640,7 +644,7 @@ L<HTML::Form>
 # ------- HTMLSelectElement interface ---------- #
 
 package HTML::DOM::Element::Select;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = 'HTML::DOM::Element';
 
 use overload fallback=>1, '@{}' => sub { shift->options };
@@ -734,7 +738,7 @@ package HTML::DOM::Collection::Options;
 use strict;
 use warnings;
 
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 
 use Carp 'croak';
 use constant::lexical sel => 5; # must not conflict with super
@@ -810,7 +814,7 @@ sub length { # override
 # ------- HTMLOptGroupElement interface ---------- #
 
 package HTML::DOM::Element::OptGroup;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = 'HTML::DOM::Element';
 
 sub label  { shift->_attr( label => @_) }
@@ -820,7 +824,7 @@ sub label  { shift->_attr( label => @_) }
 # ------- HTMLOptionElement interface ---------- #
 
 package HTML::DOM::Element::Option;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 use Carp 'croak';
@@ -931,7 +935,7 @@ sub _reset { delete shift->{_HTML_DOM_sel} }
 # ------- HTMLInputElement interface ---------- #
 
 package HTML::DOM::Element::Input;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 use Carp 'croak';
@@ -1140,7 +1144,7 @@ sub content {
 # ------- HTMLTextAreaElement interface ---------- #
 
 package HTML::DOM::Element::TextArea;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 sub defaultValue { # same as HTML::DOM::Element::Title::text
@@ -1188,7 +1192,7 @@ sub _reset {
 # ------- HTMLButtonElement interface ---------- #
 
 package HTML::DOM::Element::Button;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 *form = \&HTML::DOM::Element::Select::form;
@@ -1203,7 +1207,7 @@ sub value      { shift->attr( value       => @_) }
 # ------- HTMLLabelElement interface ---------- #
 
 package HTML::DOM::Element::Label;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 *form = \&HTML::DOM::Element::Select::form;
@@ -1213,7 +1217,7 @@ sub htmlFor { shift->_attr( for       => @_) }
 # ------- HTMLFieldSetElement interface ---------- #
 
 package HTML::DOM::Element::FieldSet;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 *form = \&HTML::DOM::Element::Select::form;
@@ -1221,7 +1225,7 @@ our @ISA = qw'HTML::DOM::Element';
 # ------- HTMLLegendElement interface ---------- #
 
 package HTML::DOM::Element::Legend;
-our $VERSION = '0.028';
+our $VERSION = '0.029';
 our @ISA = qw'HTML::DOM::Element';
 
 *form = \&HTML::DOM::Element::Select::form;
